/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Robert Olson (olson@cs.uiuc.edu)
 * Contributing Author: Ruth A. Aydt (aydt@cs.uiuc.edu)
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613 DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 * GraphEdge.h: Displays an edge on the execution graph.
 *
 * $Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/System/Includes/RCS/GraphEdge.h,v 1.12 1994/02/25 04:29:18 aydt Exp $
 *	
 */

#ifndef GraphEdge_h
#define GraphEdge_h

#include "ShapedLabel.h"

class GraphNode;

class GraphEdge : public ShapedLabel {
/*
 * Maintains and displays an edge on the execution graph.  Edges are
 * created in GraphDisplay and deleted in GraphNode when the node they
 * are connected to is destroyed.
 */

private:
	CString         edgeName;

	GraphNode	*sourceNode;

	Dimension 	maskWidth; 		// Parent"s width and height
	Dimension	maskHeight;
	
	Position startX, startY, endX, endY;    // Current endpoints for edge

	GC 		blankGC;		// Various Graphics Contexts
	GC 		lineGC;

	/* Method _setEndpoints:		 Set edge endpoints         */
	void _setEndpoints( Position sx, Position sy,
			    Position ex, Position ey );

protected:
	/* Method initializeGraphics:	Overloads ShapedLabel method.         */
	virtual void initializeGraphics();

	/* Method setupShape:		Overloads ShapedLabel method.         */
	virtual void setupShape();

public:
        /* Method GraphEdge: 		The GraphEdge constructor. 	      */
	GraphEdge( WidgetBase *par, const XArgs& args, const CString& name,
		   GraphNode *fromNode, GraphNode *toNode );

        /* Method ~GraphEdge: 		The GraphEdge destructor.             */
	~GraphEdge();

	/* Method getSourceNode:	Returns a pointer to the GraphNode
	*				that is the source for this edge.     */
	GraphNode * getSourceNode() const;

	/* Method setStart:		Set the start coordinates for edge.   */
	void setStart( Position x, Position y );

	/* Method setEnd:		Set the end coordinates for edge.     */
	void setEnd( Position x, Position y );

};

inline GraphNode *
GraphEdge::getSourceNode() const
{
	return sourceNode;
}
	
#endif
