/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Daniel A. Reed (reed@cs.uiuc.edu)
 * Contributing Author: Ruth A. Aydt (aydt@cs.uiuc.edu)
 *
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 * SynthesizeVector.h: A functional unit to synthesize a vector from scalars
 *
 *	$Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/System/Includes/RCS/SynthesizeVector.h,v 1.13 1994/03/15 16:26:23 aydt Exp $
 */

#ifndef SynthesizeVector_h
#define SynthesizeVector_h

#include "CStringObjList.h"
#include "FunctionalUnit.h"
#include "Value.h"

class MathOperation;
class InputPort;
class OutputPort;

class SynthesizeVector : public FunctionalUnit {
/*
* SynthesizeVector is a functional unit that synthesizes a vector.
* The output is always of type DOUBLE, dimension=1.  The dimension size 
* is set by the user in the configuration process.
*/

enum vectorOpTypes { ADDITION, SUBTRACTION, MULTIPLICATION,
                     DIVISION, MINIMUM, MAXIMUM, COUNT };

enum vectorChoiceTypes { REPLACE, UPDATE };

private:
	static const char *const MY_CLASS;	// My class name
	static CStringObjList	 SynthesisOpNameList;
	static CStringObjList	 SynthesisActionList;

	InputPort	*index;
	InputPort	*cellValue;
	OutputPort	*outputValue;

	MathOperation	*myOperator;
	int		myOperatorNumber;
	int		actionOption;

	Boolean_ 	isConfigured;
	Boolean_	isReady;

	Value 	 	vector;
	int      	dimension;
	int	 	dimensionMax;

	Value		resultV;
	Value		initialValue;

	/* Method _resizeVector: 	Resizes original vector to one 
	*                        	that can accomodate a larger index  */
	void _resizeVector( int newIndex );

        /* Method _setOperator:   	Set binary operator to use          */
        void _setOperator( const int opId );

public:
	/* Method SynthesizeVector:     The SynthesizeVector constructor    */
	SynthesizeVector();

	/* Method ~SynthesizeVector:    The SynthesizeVector destructor	    */
	~SynthesizeVector();

	/* Method configure:	     	Configure the functional unit       */
	virtual void configure();

	/* Method configureOperation:	Configure the action & operation    */
	virtual void configureOperation();

	/* Method copy:		     	Copy this functional un		    */
	virtual FunctionalUnit * copy();

	/* Method init:		     	Initialize the functional unit	    */
	virtual void init();

        /* Method inputTraitsValid:   	Sets traits of output port. Here
        *                            	input bindings are always valid
        *                            	so we always return TRUE_.          */
        virtual Boolean_ inputTraitsValid();

	/* Method loadConfigFromFile: 	Load configuration from a file 	    */
	virtual Boolean_ loadConfigFromFile( const CString& fileName );

	/* Method ready: 	     	Returns TRUE_ if the functional 
	*			     	unit is currently executable	    */	
	virtual Boolean_ ready();

	/* Method run:		     	Execute the functional unit to 
	*			     	fill in a cell of the vector.	    */
	virtual void run( Boolean_& errorFlag );

	/* Method saveConfigToFile:  	Save configuration to a file   	    */
	virtual Boolean_ saveConfigToFile( const CString& fileName ) const;
};

#endif SynthesizeVector_h
