# Perl script 'makeOptions.perl'
# Usage: perl makeOptions Options-File 
# writes PabloOptions.C to the stdout

($pgm = $0) =~ s,.*/,,;

@ARGV == 1 || die "Usage: $pgm options-file\n";

$optsFile = shift;

open(OPTS, "<$optsFile") || die "Can't open options file\n";
open(OUTPUT, ">&STDOUT");
chop($pwd = `pwd`);
if ($optsFile !~ m#^/#)
{
    $optsFile = "$pwd/$optsFile";
    ($base, $file) = $optsFile =~ m,(.*)/(.*),;
    chop($dir = `cd $base && pwd`);
    $optsFile = "$dir/$file";
}

chop($date = `date`);


# Write header of file
print OUTPUT <<EOF;
/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Project Manager and Principal Investigator:
 *      Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
// PabloOptions.o created from $optsFile
// Creation date: $date
// Created by: $ENV{USER}

#include "CString.h"

CString getCompiledOptions()
{
	CString str;
	CString enabled, disabled;

EOF
    
while (<OPTS>)
{
    if (($flag, $name) = /^\s*-(D|U)(\S+)/)
    {
	$which = ($flag eq "D") ? "enabled" : "disabled";

#	print "got flag '$flag' which '$which' name '$name'\n";

	print OUTPUT <<EOF;
#ifdef $name
	enabled = enabled + "    $name\\n";
#else
	disabled = disabled + "    $name\\n";
#endif
EOF
    }
}
	
print OUTPUT <<EOF;

str = "Enabled options:\\n";
str = str + enabled;
str = str + "\\n";
str = str + "Disabled options:\\n";
str = str + disabled;
str = str + "\\n";
str = str + "Options file:\\n";
str = str + "    $optsFile\\n";

return str;

}

EOF
